rm(list=ls())
library(marima2)
sapply(dir("functions/", full.names=TRUE), source)


## ---------------------------------------------------------------------------------------
# A simulation function for ARMA simulation, use model as arima.sim, i.e. flip sign of phi (into ar) coefficients
simarmax <- function(model, x, sd=1, nburnin=100){
  n <- length(x) + nburnin
  x <- c(rep(x[1],nburnin), x)
  # Take the ar and ma part
  ar <- model$ar
  ma <- model$ma
  xcoef <- model$x
  # The order (i.e. the number of lags)
  p <- length(ar)
  q <- length(ma)
  r <- length(xcoef)
  # The vector for the simulation result
  y <- numeric(n)
  # Generate the random normal values
  eps <- rnorm(n, sd=sd)
  # Run the simulation
  for(i in (max(p,q,r)+1):n){
    y[i] <- eps[i] + sum(ar * y[i-(1:p)]) + sum(xcoef * x[i-(1:r)] ) + sum(ma * eps[i-(1:q)])
  }
  # Return without the burn-in period
  return(y[(nburnin+1):n])
}


## ---------------------------------------------------------------------------------------
# Simulate an ARMAX(1,1) model, with a step input
# Test it by comparing
n <- 100
# Make a step input
x <- c(rep(0,50), rep(10,n-50))
model <- list(ar=c(0.6), ma=c(1), x=c(0.9))
set.seed(12)
y <- simarmax(model, x)
# Plot to see it
plot(y, type="b")
lines(x, type="s")


## ---------------------------------------------------------------------------------------
# Estimate with arima (must lag x1)
x1 <- lagvec(x,1)
x1[1] <- 0
(fitArima <- arima(y, order=c(1,0,1), xreg=x1, include.mean=FALSE))
# Validation ok?
resArima <- validate(fitArima)

# Estimate with marima
fitMarima <- marima("y ~ AR(1) + x(1) + MA(1)", data=data.frame(y=y,x=x))
summary(fitMarima)
# Validation ok?
resMarima <- validate(fitMarima)

# Difference in residuals
plot(resArima - resMarima)


## ---------------------------------------------------------------------------------------
# CHALLENGE: CAN YOU FIND A WAY TO FIT AN ARMAX!? (like in marima)
## ---------------------------------------------------------------------------------------

## ---------------------------------------------------------------------------------------
# First an AR model with lm
# Add the lags to X
maxlag <- 5
D <- data.frame(y=y, x=x)
for(i in 1:maxlag){
  tmp <- lagdf(D[ ,1:2], i)
  names(tmp) <- paste0(names(tmp),".l",i)
  D <- cbind(D, tmp)
}
names(D)


## ---------------------------------------------------------------------------------------
# TASK: Try making the ARMAX fit marima style with lm()
## ---------------------------------------------------------------------------------------

# First fit an ARX with lm()
fit <- lm(y ~ 0 + y.l1 + x.l1, data=D)
res <- validate(fit)

# Then repeat using the residuals


